unit uHIDPJQDevice;

interface
uses
  Windows, SysUtils, Classes, uHIDDevice;

const HID_PJQ_VID = $10C4;
const HID_PJQ_PID = $82CD;
// ----------------------------------------------------------------------------
// Report IDs
// ----------------------------------------------------------------------------
const OUT_SOUNDID =			$01; 
const OUT_KEYINID =			$02;
const OUT_SET_STARID = 		$03;
const OUT_SYS_RSTID = 			$04;
const IN_KEYCODEID =			$05; 
const IN_STATSID =		    $06; 

// ----------------------------------------------------------------------------
// Report Sizes (in bytes)
// ----------------------------------------------------------------------------
const OUT_SOUNDSize =			 $01;
const OUT_KEYINSize =			 $01; 
const OUT_SYS_RSTSize =		 $01;
const OUT_SET_STARSize =		 $01;
const IN_KEYCODESize =		 	 $01;
const IN_STATSSize =			 $01; 

const WELCOME_VOICE =           $00;
const EVALUATION_VOICE =        $01;
const THANK_VOICE =             $02;


type

  IReportable = interface
  
  end;

  THIDPJQDevice = class(TInterfacedObject, IReportable)
  private
    dev: THIDDevice;
    procedure Output(reportId, data: Byte);
    procedure PlaySound(index: Byte);
    function DoKeyIn(timeout: DWORD): Byte;
    procedure FlashKeyLed(flash: Boolean);
  public
    constructor Create;
    destructor Destroy; override;
    function IsConnected(): Boolean;
    function IsOpened(): Boolean;
    function Open(): Integer;
    function Close(): Integer;
    procedure Reset();
    procedure SetStar(level: Byte);
    function Status(): Byte;
    procedure Welcome();
    function Evaluate(timeout: DWORD): Byte;

  end;

implementation

{ THIDPJQDevice }

function THIDPJQDevice.Close: Integer;
begin
  Result := dev.Close();
end;

constructor THIDPJQDevice.Create;
begin
  dev := THIDDevice.Create;
end;

destructor THIDPJQDevice.Destroy;
begin
  dev.Free;
  inherited;
end;

function THIDPJQDevice.DoKeyIn(timeout: DWORD): Byte;
var
  inReport: array[0..2] of Byte;
  r, status: Integer;
begin
  if (dev.IsOpened()) then begin
    FlashKeyLed(True);
    try
      dev.SetTimeouts(timeout, 100);
      Result := 0;
      inReport[0] := IN_KEYCODEID;
      inReport[1] := 0;
      inReport[2] := 0;
      status := dev.GetReport_Interrupt(inReport, 3, 1, r);
      if ((status = HID_DEVICE_SUCCESS) and (inReport[0] = IN_KEYCODEID)) then begin
        Result := inReport[1];
      end;
    finally
      FlashKeyLed(False);
    end;
  end else Result := $FF;
end;

function THIDPJQDevice.Evaluate(timeout: DWORD): Byte;
begin
  if not IsOpened() then Open();
  try
    PlaySound(EVALUATION_VOICE);
    Result := DoKeyIn(timeout);
    Reset();
  finally
    Close();
  end;
end;

procedure THIDPJQDevice.FlashKeyLed(flash: Boolean);
var
  data: Byte;
begin
  data := 0;
  if flash then data := 1;
  Output(OUT_KEYINID, data);
end;

function THIDPJQDevice.IsConnected: Boolean;
begin
  Result := dev.GetConnectedDeviceNum(HID_PJQ_VID, HID_PJQ_PID) > 0;
end;

function THIDPJQDevice.IsOpened: Boolean;
begin
  Result := dev.IsOpened();
end;

function THIDPJQDevice.Open: Integer;
var
  n: Integer;
begin
  n := dev.GetConnectedDeviceNum(HID_PJQ_VID, HID_PJQ_PID);
  if (n > 0) then begin
    Result := dev.Open(n - 1, HID_PJQ_VID, HID_PJQ_PID);
    if dev.IsOpened() then dev.SetTimeouts(100, 100);
  end else Result := HID_DEVICE_NOT_FOUND;

end;

procedure THIDPJQDevice.Output(reportId, data: Byte);
var
  OutputBuffer: array[0..2] of Byte;
begin
  if (dev.IsOpened()) then begin
    OutputBuffer[0] := reportId;
    OutputBuffer[1] := data;
    OutputBuffer[2] := $FF - data;
    dev.SetReport_Interrupt(OutputBuffer, 3);
  end;
end;

procedure THIDPJQDevice.PlaySound(index: Byte);
begin
  Output(OUT_SOUNDID, index);
end;

procedure THIDPJQDevice.Reset;
begin
  Output(OUT_SYS_RSTID, 0);
end;

procedure THIDPJQDevice.SetStar(level: Byte);
begin
  if not IsOpened() then Open();
  try
    Output(OUT_SET_STARID, level);
  finally
    Close();
  end;
end;

function THIDPJQDevice.Status: Byte;
var
  inReport: array[0..2] of Byte;
  r, stat: Integer;
begin
  if not IsOpened() then Open();
  try
    inReport[0] := IN_STATSID;
    inReport[1] := 0;
    inReport[2] := 0;
    stat := dev.GetReport_Interrupt(inReport, 3, 1, r);
    Result := 0;
    if ((stat = HID_DEVICE_SUCCESS) and (inReport[0] = IN_STATSID)) then begin
      Result := inReport[1];
    end;
  finally
    Close();
  end;
end;

procedure THIDPJQDevice.Welcome;
begin
  if not IsOpened() then Open();
  try
    PlaySound(WELCOME_VOICE);
  finally
    Close();
  end;
end;

end.
