unit uCOMDevice;

interface
uses
  Windows, SysUtils, Classes, SyncObjs, Math;
  
const
  PL2303_VID = '067B';
  PL2303_PID = '2303';

  FTDI_VID = '0403';
  FTDI_PID = '6001';

type

  TCOMDevice = class
  protected
    fHandle: THandle;
    fAsyn: Boolean;
    fTimeout: Integer;
    fAutoClose: Boolean;
    fOverlapped: OVERLAPPED;
  public
    constructor Create(comPort: Integer; asyn: Boolean = false); overload;
      virtual;
    constructor Create(handle: THandle; asyn: Boolean = false); overload;
      virtual;
    constructor Create(comStr: string; asyn: Boolean = false); overload; virtual;
    destructor Destroy; override;
    function setState(baudRate: Integer = CBR_9600; byteSize: Integer = 8;
      parity: Integer = NOPARITY; stopBits: Integer = ONESTOPBIT): boolean;
    function setParity(parity: Integer): boolean;
    function setTimeout(timeout: Integer): boolean;
    function writeAddress(address: Byte): boolean;
    function writeData(const Buffer; Count: Longint): boolean;
    function write(const Buffer; Count: Longint): boolean;
    function read(var Buffer; Count: Longint): boolean;
    property Handle: THandle read fHandle;
  end;

  TPL2303Device = class
  public
    class function GetConnected(): Boolean;
    class function GetDevice(): TCOMDevice;  
  end;

  TFTDIDevice = class
  public
    class function GetConnected(): Boolean;
    class function GetDevice(): TCOMDevice;  
  end;
  
implementation
uses
  uUSBClass;
{ TCOMDevice }

constructor TCOMDevice.Create(comPort: Integer; asyn: Boolean = false);
begin
  Create(Format('\\.\COM%d', [comPort]), asyn);
end;

constructor TCOMDevice.Create(handle: THandle; asyn: Boolean = false);
begin
  fAsyn := asyn;
  fHandle := handle;
  fAutoClose := False;
  fTimeout := 0;
end;

constructor TCOMDevice.Create(comStr: string; asyn: Boolean);
begin
  fAsyn := asyn;
  fTimeout := 0;
  fAutoClose := True;
  if fAsyn then begin
    fHandle := CreateFile(PChar(comStr),
      GENERIC_READ or GENERIC_WRITE,
      0,
      nil,
      OPEN_EXISTING,
      FILE_FLAG_OVERLAPPED,
      0);
  end else begin
    fHandle := CreateFile(PChar(comStr),
      GENERIC_READ or GENERIC_WRITE,
      0,
      nil,
      OPEN_EXISTING,
      0,
      0);
  end;
end;

destructor TCOMDevice.Destroy;
begin
  if (fHandle <> INVALID_HANDLE_VALUE) and fAutoClose then
    CloseHandle(fHandle);
  inherited;
end;

function TCOMDevice.read(var Buffer; Count: Integer): boolean;
var
  c: Cardinal;
  tick: Cardinal;
  ol: OVERLAPPED;
  comstat: TComStat;
  errFlag: Cardinal;
  bTimeout: Boolean;
begin
  if fAsyn = true then begin
    FillChar(ol, SizeOf(ol), 0);
    ol.hEvent := CreateEvent(nil, True, False, nil);
    Result := ReadFile(fHandle, Buffer, Count, c, @ol);
    if not Result then begin
      if ERROR_IO_PENDING = GetLastError() then begin
        tick := GetTickCount();
        bTimeout := False;
        while not GetOverlappedResult(fHandle, ol, c, False) do begin
          if c >= Count then begin
            Result := True;
          end;
          if GetTickCount - tick >= fTimeout then begin
            bTimeout := True;
            break;
          end;
          Sleep(1);
        end;
        if not bTimeout then Result := True;
        
      end;
    end;
    CloseHandle(ol.hEvent);
  end else
    Result := ReadFile(fHandle, Buffer, Count, c, nil);
  ClearCommError(fHandle, errFlag, @comstat);
end;

function TCOMDevice.setParity(parity: Integer): boolean;
var
  dcb: TDCB;
begin
  GetCommState(Handle, dcb);
  dcb.Parity := parity;
  Result := SetCommState(Handle, dcb);
end;

function TCOMDevice.setState(baudRate, byteSize, parity, stopBits: Integer):
  boolean;
var
  dcb: TDCB;
begin
  GetCommState(Handle, dcb);
  dcb.BaudRate := baudRate;
  dcb.ByteSize := byteSize;
  dcb.Parity := parity;
  dcb.StopBits := stopBits;
  Result := SetCommState(Handle, dcb);
end;

function TCOMDevice.setTimeout(timeout: Integer): boolean;
var
  commTimeouts: TCOMMTIMEOUTS;
begin
  GetCommTimeouts(Handle, commTimeouts);
  fTimeout := timeout;
  commTimeouts.ReadIntervalTimeout := 0;
  commTimeouts.ReadTotalTimeoutMultiplier := 0;
  commTimeouts.ReadTotalTimeoutConstant := timeout;
  Result := SetCommTimeouts(Handle, commTimeouts);
end;

function TCOMDevice.write(const Buffer; Count: Integer): boolean;
var
  c: Cardinal;
  ol: OVERLAPPED;
  tick: Cardinal;
  bTimeout: Boolean;
begin
  if fAsyn = true then begin
    FillChar(ol, SizeOf(OVERLAPPED), 0);
    ol.hEvent := CreateEvent(nil, True, False, nil);
    PurgeComm(fHandle, PURGE_RXABORT or PURGE_RXCLEAR);
    Result := WriteFile(fHandle, Buffer, Count, c, @ol);
    if not Result then begin
      if ERROR_IO_PENDING = GetLastError() then begin
        tick := GetTickCount();
        bTimeout := False;
        while not GetOverlappedResult(fHandle, ol, c, False) do begin
          if c >= Count then begin
            Result := True;
          end;
          if GetTickCount - tick >= fTimeout then begin
            bTimeout := True;
            break;
          end;
          Sleep(1);
        end;
        if not bTimeout then Result := True;
        
      end;
    end;
    CloseHandle(ol.hEvent);
  end else
    Result := WriteFile(fHandle, Buffer, Count, c, nil);
end;

function TCOMDevice.writeAddress(address: Byte): boolean;
var
  pa: ^Byte;
begin
  Result := setParity(MARKPARITY);
  if Result then begin
    pa := @address;
    Result := Self.write(pa, 1);
  end;
end;

function TCOMDevice.writeData(const Buffer; Count: Integer): boolean;
begin
  Result := setParity(SPACEPARITY);
  if Result then begin
    Result := Self.write(Buffer, Count);
  end;
end;


{ TPL2303Device }

class function TPL2303Device.GetConnected: Boolean;
begin
  Result := TUSBClass.IsDeviceConnected(PL2303_VID, PL2303_PID);
end;

class function TPL2303Device.GetDevice: TCOMDevice;
var
  comPort: String;
begin
  Result := nil;
  if TUSBSerial.IsDeviceConnected(PL2303_VID, PL2303_PID, comPort) then begin
    comPort := '\\.\' + comPort;
    Result := TCOMDevice.Create(comPort);
  end;
end;

{ TFTDIDevice }

class function TFTDIDevice.GetConnected: Boolean;
begin
  Result := TUSBClass.IsDeviceConnected(FTDI_VID, FTDI_PID);
end;

class function TFTDIDevice.GetDevice: TCOMDevice;
var
  comPort: String;
begin
  Result := nil;
  if TUSBSerial.IsDeviceConnected(FTDI_VID, FTDI_PID, comPort, 'FTDIBUS') then begin
    comPort := '\\.\' + comPort;
    Result := TCOMDevice.Create(comPort);
  end;
end;

end.

