unit uHIDDevice;

interface
uses
  Windows, SysUtils, Classes, SetupApi, Hid;

const HID_DEVICE_SUCCESS				= $00;
const HID_DEVICE_NOT_FOUND			= $01;
const HID_DEVICE_NOT_OPENED			= $02;
const HID_DEVICE_ALREADY_OPENED		= $03;
const	HID_DEVICE_TRANSFER_TIMEOUT		= $04;
const HID_DEVICE_TRANSFER_FAILED		= $05;
const HID_DEVICE_CANNOT_GET_HID_INFO	= $06;
const HID_DEVICE_HANDLE_ERROR			= $07;
const HID_DEVICE_INVALID_BUFFER_SIZE	= $08;
const HID_DEVICE_SYSTEM_CODE			= $09;
const HID_DEVICE_UNKNOWN_ERROR		= $FF;

// Max number of USB Devices allowed
const MAX_USB_DEVICES	= 64;

// Max number of reports that can be requested at time
const MAX_REPORT_REQUEST_XP =	512;
const MAX_REPORT_REQUEST_2K	= 200;

const DEFAULT_REPORT_INPUT_BUFFERS = 0;

// String Types
const HID_VID_STRING				= $01;
const HID_PID_STRING				= $02;
const HID_PATH_STRING				= $03;
const HID_SERIAL_STRING			= $04;

// String Lengths
const VID_LENGTH				 = 5;
const PID_LENGTH				 = 5;
const MAX_PATH_LENGTH		 = MAX_PATH;
const MAX_SERIAL_STRING_LENGTH = 256;

type

  THIDDevice = class
  private
    fDeviceOpened: Boolean;
    fHandle: THandle;
    fDevicePathString: String;
    fGetReportTimeout: DWORD;
    fSetReportTimeout: DWORD;
    fInputReportBufferLength: Integer;
    fOutputReportBufferLength: Integer;
    fFeatureReportBufferLength: Integer;
    fMaxReportRequest: Integer;

	  procedure ResetDeviceData();
  public
    constructor Create;
    destructor Destroy; override;

    function Open(deviceIndex: Integer; vid, pid: WORD;
        numInputBuffers: WORD = DEFAULT_REPORT_INPUT_BUFFERS): Integer;
	  function IsOpened(): Boolean;

    function GetInputReportBufferLength(): Integer;
    function GetOutputReportBufferLength(): Integer;
    function GetFeatureReportBufferLength(): Integer;
    function GetMaxReportRequest(): Integer;
  	function FlushBuffers(): Boolean;
//
  	procedure GetTimeouts(var getReportTimeout: Integer; var setReportTimeout: Integer);
	  procedure SetTimeouts(const getReportTimeout: Integer; const setReportTimeout: Integer);
//
    function Close(): Integer;
    class function GetHidDevicePath(const index: Integer; var devicePath: String): Boolean;
    class function OpenDeviceByVidPid(const devicePath: String;const vid, pid: Integer): THandle;
    class function GetConnectedDeviceNum(const vid, pid: Integer): Integer;
    class function GetSerialString(const deviceIndex: Integer;const vid, pid: Integer;
      var serialString: String): Integer;
    function SetFeatureReport(var buffer; bufferSize: Integer): Integer;
    function GetFeatureReport(var buffer; bufferSize: Integer): Integer;
    function SetReport_Interrupt(const buffer; bufferSize: Integer): Integer;
    function GetReport_Interrupt(var buffer; bufferSize: Integer; numReports: Integer; var bytesReturned: Integer): Integer;
    function SetReport_Control(const buffer; bufferSize: Integer): Integer;
    function GetReport_Control(var buffer; bufferSize: Integer): Integer;
  end;

implementation

{ THIDDevice }

function THIDDevice.Close: Integer;
begin
  Result := HID_DEVICE_SUCCESS;
  if (fDeviceOpened) then begin
    if ((fHandle <> INVALID_HANDLE_VALUE) and (fHandle <> 0)) then begin
      CloseHandle(fHandle);
    end else Result := HID_DEVICE_HANDLE_ERROR;
    ResetDeviceData();
  end else begin
    Result := HID_DEVICE_NOT_OPENED;
  end;
end;

constructor THIDDevice.Create;
begin
  fGetReportTimeout := INFINITE;
  fSetReportTimeout := INFINITE;
  ResetDeviceData();
end;

destructor THIDDevice.Destroy;
begin
  Close();
  inherited;
end;

function THIDDevice.FlushBuffers: Boolean;
begin
  Result := HidD_FlushQueue(fHandle);
end;

class function THIDDevice.GetConnectedDeviceNum(const vid, pid: Integer): Integer;
var
  devicePath: String;
  i: Integer;
  h: THandle;
begin
  Result := 0;
  i := 0;
  while GetHidDevicePath(i, devicePath) do begin
    h := OpenDeviceByVidPid(devicePath, vid, pid);
    if (h <> INVALID_HANDLE_VALUE) then begin
      Inc(Result);
      CloseHandle(h);
    end;
    Inc(i);
  end;
end;

function THIDDevice.GetFeatureReport(var buffer; bufferSize: Integer): Integer;
var
  reportId: Byte;
begin
  Result := HID_DEVICE_SUCCESS;
  if (IsOpened()) then begin
    reportId := PByte(@buffer)^;
    FillChar(buffer, bufferSize, 0);
    PByte(@buffer)^ := reportId;
    if (not HidD_GetFeature(fHandle, buffer, bufferSize)) then
      Result := HID_DEVICE_TRANSFER_FAILED;
  end else begin
    Result := HID_DEVICE_NOT_OPENED;
  end;

end;

function THIDDevice.GetFeatureReportBufferLength: Integer;
begin
  Result := fFeatureReportBufferLength;
end;

class function THIDDevice.GetHidDevicePath(const index: Integer;
  var devicePath: String): Boolean;
var
  hidGuid: TGUID;
  devInfo: HDEVINFO;
  intfData: TSPDeviceInterfaceData;
  hidResult, detailResult: Boolean;
  len, required: DWORD;
  p_detailData: PSPDeviceInterfaceDetailData;
begin
  Result := False;
  HidD_GetHidGuid(hidGuid);
  devInfo := SetupDiGetClassDevs(@hidGuid, nil, 0, DIGCF_PRESENT or DIGCF_DEVICEINTERFACE);
  if (devInfo <> nil) then begin
    try
      if (index < MAX_USB_DEVICES) then begin
        FillChar(intfData, sizeof(intfData), 0);
        intfData.cbSize := sizeof(intfData);
        hidResult := SetupDiEnumDeviceInterfaces(devInfo, nil, hidGuid, index, intfData);
        if (hidResult) then begin
          SetupDiGetDeviceInterfaceDetail(devInfo, @intfData, nil, 0, len, nil);
          GetMem(p_detailData, len);
          try
            FillChar(p_detailData^, len, 0);
            p_detailData^.cbSize := sizeof(TSPDeviceInterfaceDetailData);
            detailResult := SetupDiGetDeviceInterfaceDetail(devInfo, @intfData,
              p_detailData, len, required, nil);
            if (detailResult) then begin
              SetLength(devicePath, len - sizeof(Cardinal));
              StrMove(PChar(devicePath), p_detailData^.DevicePath, len - sizeof(Cardinal));
              Result := True;
            end;
          finally
            FreeMem(p_detailData);
          end;

        end;
      end;
    finally
      SetupDiDestroyDeviceInfoList(devInfo);
    end;
  end;
end;

function THIDDevice.GetInputReportBufferLength: Integer;
begin
  Result := fInputReportBufferLength;
end;

function THIDDevice.GetMaxReportRequest: Integer;
begin
  Result := 32;
  if fMaxReportRequest > 0 then Result := fMaxReportRequest;
end;

function THIDDevice.GetOutputReportBufferLength: Integer;
begin
  Result := fOutputReportBufferLength;
end;

function THIDDevice.GetReport_Control(var buffer; bufferSize: Integer): Integer;
var
  reportId: Byte;
begin
  Result := HID_DEVICE_SUCCESS;
  if (bufferSize >= Self.fInputReportBufferLength) then begin
    if (IsOpened()) then begin
      reportId := PByte(@buffer)^;
      FillChar(buffer, bufferSize, 0);
      PByte(@buffer)^ := reportId;
      if (not HidD_GetInputReport(fHandle, @buffer, bufferSize)) then
        Result := HID_DEVICE_TRANSFER_FAILED;
    end else begin
      Result := HID_DEVICE_NOT_OPENED;
    end;
  end else begin
    Result := HID_DEVICE_INVALID_BUFFER_SIZE;
  end;
end;

function THIDDevice.GetReport_Interrupt(var buffer; bufferSize,
  numReports: Integer; var bytesReturned: Integer): Integer;
var
  bytesRead: Cardinal;
  o: TOverlapped;
  waitStatus: Cardinal;
begin
  Result := HID_DEVICE_SUCCESS;
  if ((bufferSize >= (fInputReportBufferLength * numReports))
    and (numReports <= fMaxReportRequest)) then begin
    FillChar(buffer, bufferSize, 0);
    if (IsOpened()) then begin
      FillChar(o, Sizeof(o), 0);
      o.hEvent := CreateEvent(nil, False, False, nil);
      if (not ReadFile(fHandle, buffer, fInputReportBufferLength * numReports, bytesRead, @o)) then begin
        if (GetLastError() = ERROR_IO_PENDING) then begin
          waitStatus := WaitForSingleObject(o.hEvent, fGetReportTimeout);
          if (waitStatus = WAIT_OBJECT_0) then begin
            GetOverlappedResult(fHandle, o, bytesRead, False);
          end else if (waitStatus = WAIT_TIMEOUT) then begin
            Result := HID_DEVICE_TRANSFER_TIMEOUT;
            CancelIo(fHandle);
          end else begin
            Result := HID_DEVICE_TRANSFER_FAILED;
            CancelIo(fHandle);
          end;
        end else Result := HID_DEVICE_TRANSFER_FAILED;
      end;
      CloseHandle(o.hEvent);
    end else Result := HID_DEVICE_NOT_OPENED;
  end else Result := HID_DEVICE_INVALID_BUFFER_SIZE;
  if (Result = HID_DEVICE_SUCCESS) then begin
    bytesReturned := bytesRead;
  end;
end;

class function THIDDevice.GetSerialString(const deviceIndex, vid, pid: Integer;
  var serialString: String): Integer;
begin
  Result := HID_DEVICE_SUCCESS;
end;

procedure THIDDevice.GetTimeouts(var getReportTimeout,
  setReportTimeout: Integer);
begin
  getReportTimeout := Self.fGetReportTimeout;
  setReportTimeout := Self.fSetReportTimeout;
end;

function THIDDevice.IsOpened: Boolean;
begin
  if ((fDeviceOpened) and (fHandle <> INVALID_HANDLE_VALUE) and (fHandle <> 0)) then begin
    Result := True;
  end else begin
    if (fDeviceOpened) then begin
      ResetDeviceData();
    end;
    Result := False;
  end;
end;

function THIDDevice.Open(deviceIndex: Integer; vid, pid,
  numInputBuffers: WORD): Integer;
var
  devicePath: String;
  i, deviceNum: Integer;
  found: Boolean;
  h: THandle;
  preparsedData: PHIDPPreparsedData;
  caps: THIDPCaps;
begin
  found := False;
  Result := HID_DEVICE_SUCCESS;
  if (fDeviceOpened) then Result := HID_DEVICE_ALREADY_OPENED;
  if(Result = HID_DEVICE_SUCCESS) then begin
    deviceNum := 0; i := 0;
    while GetHidDevicePath(i, devicePath) do begin
      h := OpenDeviceByVidPid(devicePath, vid, pid);
      if (h <> INVALID_HANDLE_VALUE) then begin
        if (deviceNum = deviceIndex) then begin
          found := True;
          fDeviceOpened := True;
          fHandle := h;

          if (HidD_GetPreparsedData(h, preparsedData)) then begin
            if(HidP_GetCaps(preparsedData, caps) > 0) then begin
              if (caps.InputReportByteLength > 0) then
                fInputReportBufferLength := caps.InputReportByteLength;
              if (caps.OutputReportByteLength > 0) then
                fOutputReportBufferLength := caps.OutputReportByteLength;
              if (caps.FeatureReportByteLength > 0) then
                fFeatureReportBufferLength := caps.FeatureReportByteLength;

              if (fMaxReportRequest > 0) and
                (numInputBuffers <> DEFAULT_REPORT_INPUT_BUFFERS) then begin
                if (numInputBuffers > fMaxReportRequest) then fMaxReportRequest := numInputBuffers;
                HidD_SetNumInputBuffers(h, numInputBuffers);
              end else begin
                HidD_GetNumInputBuffers(h, fMaxReportRequest);
              end;
            end;
            HidD_FreePreparsedData(preparsedData);
          end;
        end;
        Inc(deviceNum);
        if(not found) then CloseHandle(h);
      end;
      Inc(i);
    end;
  end;
end;

class function THIDDevice.OpenDeviceByVidPid(const devicePath: String; const vid,
  pid: Integer): THandle;
var
  attr: THIDDAttributes;
begin
  Result := CreateFile(PChar(devicePath), GENERIC_READ or GENERIC_WRITE, 0, nil,
    OPEN_EXISTING, FILE_FLAG_OVERLAPPED, 0);

  if (Result <> INVALID_HANDLE_VALUE) then begin
    if (HidD_GetAttributes(Result, attr)) then begin
      if ((attr.VendorID <> vid) or (attr.ProductID <> pid)) then begin
        CloseHandle(Result);
        Result := INVALID_HANDLE_VALUE;
      end;
    end else begin
      CloseHandle(Result);
      Result := INVALID_HANDLE_VALUE;
    end;
  end;
end;

procedure THIDDevice.ResetDeviceData;
var
  osVer: TOSVersionInfo;
begin
  fDeviceOpened := False;
  fHandle := INVALID_HANDLE_VALUE;
  fDevicePathString := '';
  fInputReportBufferLength := 0;
  fOutputReportBufferLength := 0;
  fFeatureReportBufferLength := 0;

  osVer.dwOSVersionInfoSize := sizeof(TOSVersionInfo);
  GetVersionEx(osVer);
	// Only set the max report requests if we are using 2K or later
	if ((osVer.dwPlatformId = 2) and (osVer.dwMajorVersion >= 5)) then begin
    if (osVer.dwMajorVersion > 5) or
      ((osVer.dwMajorVersion = 5) and (osVer.dwMinorVersion >= 1)) then begin
			// XP or later supports 512 input reports
			fMaxReportRequest := MAX_REPORT_REQUEST_XP;
		end else begin
			// 2K supports 200 input reports
			fMaxReportRequest := MAX_REPORT_REQUEST_2K;
		end;
	end else begin
		// Otherwise set the max reports to 0, and this will
		// only allow te default
		fMaxReportRequest := 0;
	end;
end;

function THIDDevice.SetFeatureReport(var buffer;
  bufferSize: Integer): Integer;
begin
  Result := HID_DEVICE_SUCCESS;
  if (IsOpened()) then begin
    if (not HidD_SetFeature(fHandle, buffer, bufferSize)) then begin
      Result := HID_DEVICE_TRANSFER_FAILED;
    end;
  end else begin
    Result := HID_DEVICE_NOT_OPENED;
  end;

end;

function THIDDevice.SetReport_Control(const buffer;
  bufferSize: Integer): Integer;
begin
  Result := HID_DEVICE_SUCCESS;
  if (IsOpened()) then begin
    if (not HidD_SetOutputReport(fHandle, @buffer, bufferSize)) then begin
      Result := HID_DEVICE_TRANSFER_FAILED;
    end;
  end else begin
    Result := HID_DEVICE_NOT_OPENED;
  end;

end;

function THIDDevice.SetReport_Interrupt(const buffer;
  bufferSize: Integer): Integer;
var
  bytesWritten: Cardinal;
  o: TOverlapped;
  waitStatus: Cardinal;
begin
  Result := HID_DEVICE_SUCCESS;
  if (bufferSize <= fOutputReportBufferLength) then begin
    if (IsOpened()) then begin
      FillChar(o, Sizeof(o), 0);
      o.hEvent := CreateEvent(nil, False, False, nil);
      if (not WriteFile(fHandle, buffer, bufferSize, bytesWritten, @o)) then begin
        if (GetLastError() = ERROR_IO_PENDING) then begin
          waitStatus := WaitForSingleObject(o.hEvent, fSetReportTimeout);
          if (waitStatus = WAIT_OBJECT_0) then begin
            GetOverlappedResult(fHandle, o, bytesWritten, False);
          end else if (waitStatus = WAIT_TIMEOUT) then begin
            Result := HID_DEVICE_TRANSFER_TIMEOUT;
            CancelIo(fHandle);
          end else begin
            Result := HID_DEVICE_TRANSFER_FAILED;
            CancelIo(fHandle);
          end;
        end else Result := HID_DEVICE_TRANSFER_FAILED;
      end;
      CloseHandle(o.hEvent);
    end else Result := HID_DEVICE_NOT_OPENED;
  end else Result := HID_DEVICE_INVALID_BUFFER_SIZE;
end;

procedure THIDDevice.SetTimeouts(const getReportTimeout,
  setReportTimeout: Integer);
begin
  Self.fGetReportTimeout := getReportTimeout;
  Self.fSetReportTimeout := setReportTimeout;
end;

initialization
  LoadHid();
  LoadSetupApi();

finalization
  UnloadSetupApi();
  UnloadHid();

end.
